@@ -6,6 +6,7 @@ stopifnot(suppressPackageStartupMessages(require(Matrix)))
 
 source(system.file("test-tools.R", package = "Matrix"), keep.source = FALSE)
 ##-> identical3() etc
+cat("doExtras:",doExtras,"\n")
 
 if(interactive()) {
     options(error = recover, warn = 1)
@@ -14,7 +15,7 @@ if(interactive()) {
 } else {
     options(Matrix.verbose = TRUE, warn = 1)
 }
-
+
 
 ### Dense Matrices
 
@@ -160,9 +161,9 @@ m0 <- Matrix(m, nrow = 40)
 m1 <- add.simpleDimnames(m0)
 for(kind in c("n", "l", "d")) {
  for(m in list(m0,m1)) { ## -- with and without dimnames -------------------------
-    kClass <- paste(kind, "Matrix", sep="")
-    Ckind <- paste(kind, "gCMatrix", sep="")
-    Tkind <- paste(kind, "gTMatrix", sep="")
+    kClass <-paste0(kind, "Matrix"  )
+    Ckind <- paste0(kind, "gCMatrix")
+    Tkind <- paste0(kind, "gTMatrix")
     str(mC <- as(m, Ckind))
     str(mT <- as(as(as(m, kClass), "TsparseMatrix"), Tkind))
     mm <- as(mC, "matrix") # also logical or double
@@ -199,7 +200,7 @@ for(kind in c("n", "l", "d")) {
     ##
     ## set.seed(7)
     cat(" for(): ")
-    for(n in 1:50) {
+    for(n in 1:(if(doExtras) 50 else 5)) {
         chkAssign(mC, mm)
         chkAssign(mC[-3,-2], mm[-3,-2])
         cat(".")
@@ -210,6 +211,93 @@ for(kind in c("n", "l", "d")) {
 }## end{for}---------------------------------------------------------------
 showProc.time()
 
+if(doExtras) {### {was ./AAA_index.R, MM-only}
+    ## an nsparse-example
+    A <- Matrix(c(rep(c(1,0,0),2), rep(c(2,0),7), c(0,0,2), rep(0,4)), 3,9)
+    i <- c(3,1:2)
+    j <- c(3, 5, 9, 5, 9)
+    vv <- logical(length(i)*length(j)); vv[6:9] <- TRUE
+
+    print(An <- as(A,"nMatrix")); an <- as(An, "matrix")
+    assert.EQ.mat(An, an)
+    An[i, j] <- vv
+    an[i, j] <- vv
+    assert.EQ.mat(An, an)## error
+    if(!all(An == an)) show(drop0(An - an))
+    ## all are +1
+
+    options("Matrix.subassign.verbose" = TRUE)# output from C
+    An <- as(A,"nMatrix"); An[i, j] <- vv
+    ## and compare with this:
+    Al <- as(A,"lMatrix"); Al[i, j] <- vv
+    options("Matrix.subassign.verbose" = FALSE)
+
+    ##--- An interesting not small not large example  for  M[i,j] <- v ------------
+    ##
+    M <- Matrix(c(1, rep(0,7), 1:4), 3,4)
+    N0 <- kronecker(M,M)
+    mkN1 <- function(M) {
+        stopifnot(length(d <- dim(M)) == 2)
+        isC <- is(M,"CsparseMatrix")
+        M[,d[2]] <- c(0,2,0)
+        N <- kronecker(diag(x=1:2), M)## remains sparse if 'M' is
+        if(isC) N <- as(N, "CsparseMatrix")
+        diag(N[-1,]) <- -2
+        N[9,]  <- 1:4   # is recycled
+        N[,12] <- -7:-9 # ditto
+        N
+    }
+
+    show(N1 <- t(N <- mkN1(N0)))    # transpose {for display reasons}
+    C1 <- t(C <- mkN1(as(N0,"CsparseMatrix")))
+    stopifnot(all(C == N))
+    assert.EQ.mat(C, mkN1(as.matrix(N0)))
+
+    C. <- C1
+    show(N <- N1) ; n <- as.matrix(N); str(N)
+    sort(i <- c(6,8,19,11,21,20,10,7,12,9,5,18,17,22,13))## == c(5:13, 17:22))
+    sort(j <- c(3,8,6,15,10,4,14,13,16,2,11,17,7,5))## == c(2:8, 10:11, 13:17)
+    val <- v.l <- 5*c(0,6,0,7,0,0,8:9, 0,0)
+    show(spv <- as(val, "sparseVector")); str(spv)
+
+    n [i,j] <- v.l
+    N [i,j] <- val# is recycled, too
+    C.[i,j] <- val
+    assert.EQ.mat(N,n) ; stopifnot(all(C. == N))
+    ## and the same *again*:
+    n [i,j] <- v.l
+    N [i,j] <- val
+    C.[i,j] <- val
+    assert.EQ.mat(N,n)
+    stopifnot(all(C. == N))
+
+    print(load(system.file("external", "symA.rda", package="Matrix"))) # "As"
+    stopifnot(isValid(As, "dsCMatrix"), identical(As@factors, list()))
+    R. <- drop0(chol(As))
+    stopifnot(1:32 == sort(diag(R.)), ## !
+              R.@x == as.integer(R.@x),## so it is an integer-valued chol-decomp !
+              ## shows that (1) As is *not* singular  (2) the matrix is not random
+              all.equal(crossprod(R.), As, tol=1e-15))
+    print(summary(evA <- eigen(As, only.values=TRUE)$val))
+    print(tail(evA)) ## largest three ~= 10^7,  smallest two *negative*
+    print(rcond(As)) # 1.722 e-21 == very bad !
+    ##-> this *is* a border line case, i.e. very close to singular !
+    ## and also determinant(.) is rather random here!
+    cc0 <- Cholesky(As)# no problem
+    try({
+        cc <- Cholesky(As, super=TRUE)
+        ## gives --on 32-bit only--
+        ## Cholmod error 'matrix not positive definite' at file:../Supernodal/t_cholmod_super_numeric.c, line 613
+        ecc <- expand(cc)
+        L.P <- with(ecc, crossprod(L,P))  ## == L'P
+        ## crossprod(L.P) == (L'P)' L'P == P'LL'P
+        stopifnot( all.equal(crossprod(L.P), As) )
+    })
+    ##---- end{ eigen( As ) -----------
+
+} ## only if(doExtras)
+
+
 ##---- Symmetric indexing of symmetric Matrix ----------
 m. <- mC
 m.[, c(2, 7:12)] <- 0
@@ -234,7 +322,7 @@ stopifnot(as.logical(grep("indices.*sparse Matrices", er$message)))
 
 N <- nrow(T)
 set.seed(11)
-for(n in 1:50) {
+for(n in 1:(if(doExtras) 50 else 3)) {
     i <- sample(N, max(2, sample(N,1)), replace = FALSE)
     validObject(Tii <- T[i,i]) ; tTi <- t(T)[i,i]
     stopifnot(is(Tii, "dsTMatrix"), # remained symmetric Tsparse
@@ -268,7 +356,7 @@ diag(A.) <- 10 * (1:6)
 a. <- as(A., "matrix")
 ## More testing {this was not working for a long time..}
 set.seed(1)
-for(n in 1:100) {
+for(n in 1:(if(doExtras) 100 else 6)) {
     i <- sample(1:nrow(A), 3+2*rpois(1, lam=3), replace=TRUE)
     Aii  <- A[i,i]
     A.ii <- A.[i,i]
@@ -371,7 +459,7 @@ Nn <- as(N0, "nMatrix"); nn <- as(Nn,"matrix")
 
 set.seed(1)
 Nn0 <- Nn00; nn0 <- nn00
-for(i in 1:200) {
+for(i in 1:(if(doExtras) 200 else 25)) {
     Nn <- Nn0
     nn <- nn0
     i. <- getDuplIndex(nrow(N0), 6)
@@ -409,8 +497,8 @@ m0 <- Diagonal(5)
 stopifnot(identical(m0[2,], m0[,2]),
 	  identical(m0[,1], c(1,0,0,0,0)))
 ### Diagonal -- Sparse:
-(m1 <- as(m0, "TsparseMatrix")) # dtTMatrix
-(m2 <- as(m0, "CsparseMatrix")) # dtCMatrix
+(m1 <- as(m0, "TsparseMatrix")) # dtTMatrix unitriangular
+(m2 <- as(m0, "CsparseMatrix")) # dtCMatrix unitriangular
 m1g <- as(m1, "generalMatrix")
 stopifnot(is(m1g, "dgTMatrix"))
 assert.EQ.mat(m2[1:3,],    diag(5)[1:3,])
@@ -425,7 +513,9 @@ uTr[1,] <- 0
 assert.EQ.mat(uTr, cbind(0, rbind(0,diag(2))))
 
 M <- m0; M[1,] <- 0
-stopifnot(identical(M, Diagonal(x=c(0, rep(1,4)))))
+Z <- m0; Z[] <- 0; z <- array(0, dim(M))
+stopifnot(identical(M, Diagonal(x=c(0, rep(1,4)))),
+          all(Z == 0), Qidentical(as(Z, "matrix"), z))
 M <- m0; M[,3] <- 3 ; M ; stopifnot(is(M, "sparseMatrix"), M[,3] == 3)
 checkMatrix(M)
 M <- m0; M[1:3, 3] <- 0 ;M
@@ -435,19 +525,23 @@ stopifnot(identical(M, Diagonal(x=c(1,1, 0, 1,1))),
 
 M <- m1; M[1,] <- 0 ; M ; assert.EQ.mat(M, diag(c(0,rep(1,4))), tol=0)
 M <- m1; M[,3] <- 3 ; stopifnot(is(M,"sparseMatrix"), M[,3] == 3)
+Z <- m1; Z[] <- 0
 checkMatrix(M)
 M <- m1; M[1:3, 3] <- 0 ;M
 assert.EQ.mat(M, diag(c(1,1, 0, 1,1)), tol=0)
 T <- m1; T[1:3, 3] <- 10; checkMatrix(T)
-stopifnot(is(T, "triangularMatrix"), identical(T[,3], c(10,10,10,0,0)))
+stopifnot(is(T, "triangularMatrix"), identical(T[,3], c(10,10,10,0,0)),
+	  Qidentical(as(Z, "matrix"), z))
 
 M <- m2; M[1,] <- 0 ; M ; assert.EQ.mat(M, diag(c(0,rep(1,4))), tol=0)
 M <- m2; M[,3] <- 3 ; stopifnot(is(M,"sparseMatrix"), M[,3] == 3)
 checkMatrix(M)
+Z <- m2; Z[] <- 0
 M <- m2; M[1:3, 3] <- 0 ;M
 assert.EQ.mat(M, diag(c(1,1, 0, 1,1)), tol=0)
 T <- m2; T[1:3, 3] <- 10; checkMatrix(T)
-stopifnot(is(T, "dtCMatrix"), identical(T[,3], c(10,10,10,0,0)))
+stopifnot(is(T, "dtCMatrix"), identical(T[,3], c(10,10,10,0,0)),
+	  Qidentical(as(Z, "matrix"), z))
 showProc.time()
 
 
@@ -471,7 +565,7 @@ M[i] <- v; assert.EQ.mat(M,m) # dge
 D[i] <- v; assert.EQ.mat(D,m) # ddi -> dtT -> dgT
 s[i] <- v; assert.EQ.mat(s,m) # dtT -> dgT
 S[i] <- v; assert.EQ.mat(S,m); S # dtC -> dtT -> dgT -> dgC
-stopifnot(identical(s,D))
+stopifnot(Q.C.identical(D,s, checkClass=FALSE))
 ## logical
 eval(.iniDiag.example)
 m[L] <- z
@@ -572,7 +666,7 @@ mc0 <- mc
 mt0 <- as(mc0, "TsparseMatrix")
 m0  <- as(mc0, "matrix")
 set.seed(1); options(Matrix.verbose = FALSE)
-for(i in 1:50) {
+for(i in 1:(if(doExtras) 50 else 4)) {
     mc <- mc0; mt <- mt0 ; m <- m0
     ev <- 1:5 %% 2 == round(runif(1))# 0 or 1
     j <- sample(ncol(mc), 1 + round(runif(1)))
@@ -695,7 +789,7 @@ showProc.time()
 m.[ cbind(3:5, 1:3) ] <- 1:3
 stopifnot(m.[3,1] == 1, m.[4,2] == 2)
 nt. <- nt ; nt[rbind(2:3, 3:4, c(3,3))] <- FALSE
-s. <- m. ; m.[cbind(3:4,2:3)] <- 0 ## assigning 0 where there *is* 0 ..
+s. <- m. ; m.[cbind(3,4:6)] <- 0 ## assigning 0 where there *is* 0 ..
 stopifnot(identical(nt.,nt),       ## should not have changed
 	  identical(s., m.))
 x.x[ cbind(2:6, 2:6)] <- 12:16
@@ -807,6 +901,7 @@ validObject(spCol <- f[,5000, drop=FALSE])
 ## *not* identical(): as(spCol, "sparseVector")@length is "double"prec:
 stopifnot(all.equal(as(spCol, "sparseVector"),
                     as(sv,   "nsparseVector"), tol=0))
+if(doExtras) {#-----------------------------------------------------------------
 f[,5762] <- thisCol # now "fine" <<<<<<<<<< FIXME uses LARGE objects -- slow --
 ## is using  replCmat() in ../R/Csparse.R, then
 ##           replTmat() in ../R/Tsparse.R
@@ -829,8 +924,11 @@ cat("checkMatrix() of all: \n---------\n")
 Sys.setlocale("LC_COLLATE", "C")# to keep ls() reproducible
 for(nm in ls()) if(is(.m <- get(nm), "Matrix")) {
     cat(nm, "\n")
-    checkMatrix(.m, verbose = FALSE)
+    checkMatrix(.m, verbose = FALSE
+		, doDet = nm != "As" ## <- "As" almost singular <=> det() "ill posed"
+		)
 }
 showProc.time()
+}#--------------end if(doExtras) -----------------------------------------------
 
 if(!interactive()) warnings()
