@@ -307,6 +307,79 @@ str(r2. <- v2 %*% M)
 stopifnot(identical3(r2, r2., t(as(v2, "matrix")) %*% M))
 
 
+###------------------------------------------------------------------
+### Close to singular matrix W
+### (from  micEconAids/tests/aids.R ... priceIndex = "S" )
+(load(system.file("external", "symW.rda", package="Matrix"))) # "symW"
+stopifnot(is(symW, "symmetricMatrix"))
+n <- nrow(symW)
+I <- .sparseDiagonal(n, shape="g")
+S <- as(symW, "matrix")
+sis <- solve(S,    S)
+## solve(<dsCMatrix>, <sparseMatrix>) when Cholmod fails was buggy for *long*:
+SIS <- solve(symW, symW)
+iw <- solve(symW)
+iss <- iw %*% symW
+##                                     nb-mm3
+assert.EQ.(I, drop0(sis), tol = 1e-9)# 2.6e-10
+assert.EQ.(I, SIS,        tol = 1e-7)# 8.2e-9
+assert.EQ.(I, iss,        tol = 4e-4)# 3.3e-5
+## solve(<dsCMatrix>, <dense..>) :
+I <- diag(nr=n)
+SIS <- solve(symW, as(symW,"denseMatrix"))
+iw  <- solve(symW, I)
+iss <- iw %*% symW
+assert.EQ.mat(SIS, I, tol = 1e-7, giveRE=TRUE)
+assert.EQ.mat(iss, I, tol = 4e-4, giveRE=TRUE)
+rm(SIS,iss)
+
+WW <- as(symW, "generalMatrix") # the one that gave problems
+IW <- solve(WW)
+class(I1 <- IW %*% WW)# "dge" or "dgC" (!)
+class(I2 <- WW %*% IW)
+## these two were wrong for for M.._1.0-13:
+assert.EQ.(as(I1,"matrix"), I, tol = 1e-4)
+assert.EQ.(as(I2,"matrix"), I, tol = 7e-7)
+
+## now slightly perturb WW (and hence break exact symmetry
+set.seed(131); ii <- sample(length(WW), size= 100)
+WW[ii] <- WW[ii] * (1 + 1e-7*runif(100))
+SW. <- symmpart(WW)
+SW2 <- Matrix:::forceSymmetric(WW)
+stopifnot(all.equal(as(SW.,"matrix"),
+                    as(SW2,"matrix"), tol = 1e-7))
+(ch <- all.equal(WW, as(SW.,"dgCMatrix"), tol=0))
+stopifnot(is.character(ch), length(ch) == 1)## had length(.)  2  previously
+IW <- solve(WW)
+class(I1 <- IW %*% WW)# "dge" or "dgC" (!)
+class(I2 <- WW %*% IW)
+I <- diag(nr=nrow(WW))
+stopifnot(all.equal(as(I1,"matrix"), I, check.attributes=FALSE, tol = 1e-4),
+          ## "Mean relative difference: 3.296549e-05"  (or "1.999949" for Matrix_1.0-13 !!!)
+          all.equal(as(I2,"matrix"), I, check.attributes=FALSE)) #default tol gives "1" for M.._1.0-13
+
+if(doExtras) {
+    print(kappa(WW)) ## [1] 5.129463e+12
+    print(rcond(WW)) ## [1] 6.216103e-14
+    ## Warning message: rcond(.) via sparse -> dense coercion
+}
+
+class(Iw. <- solve(SW.))# FIXME? should be "symmetric" but is not
+class(Iw2 <- solve(SW2))# FIXME? should be "symmetric" but is not
+class(IW. <- as(Iw., "denseMatrix"))
+class(IW2 <- as(Iw2, "denseMatrix"))
+
+### The next two were wrong for very long, too
+assert.EQ.(I, as.matrix(IW. %*% SW.), tol= 4e-4)
+assert.EQ.(I, as.matrix(IW2 %*% SW2), tol= 4e-4)
+dIW <- as(IW, "denseMatrix")
+assert.EQ.(dIW, IW., tol= 4e-4)
+assert.EQ.(dIW, IW2, tol= 8e-4)
+
+##------------------------------------------------------------------
+
+
+
 ## Sparse Cov.matrices from  Harri Kiiveri @ CSIRO
 a <- matrix(0,5,5)
 a[1,2] <- a[2,3] <- a[3,4] <- a[4,5] <- 1
@@ -326,7 +399,8 @@ assert.EQ.mat(B2 %*% z.s, as(bb, "matrix"))
 ## -> dense RHS and dense result
 z. <- solve(as(B2, "dgCMatrix"), bb)# now *sparse*
 z  <- solve( B2, as(bb,"dgeMatrix"))
-stopifnot(is(z., "sparseMatrix"), all.equal(z, as(z.,"denseMatrix")))
+stopifnot(is(z., "sparseMatrix"),
+          all.equal(z, as(z.,"denseMatrix")))
 ## finish calculating conditional variance matrix
 v <- b[3:5,3:5] - crossprod(bb,z)
 stopifnot(all.equal(as.mat(v),
